#import "DirectoryBrowserDataSource.h"
#import "DirectoryBrowserItem.h"
#import "../Utils/Utils.h"


@implementation DirectoryBrowserDataSource

@synthesize directoryBrowserView = mDirectoryBrowserView;

-(id)init
{
	DEBUG_LOG(@"DirectoryBrowserDataSource - init");

	self = [super init];
	if (self != nil)
	{
		// create the root directories
		mRootDirectories = [[NSMutableArray alloc] init];
		DirectoryBrowserItem * home = [[DirectoryBrowserItem alloc] initWithPath:NSHomeDirectory()];
		DirectoryBrowserItem * volumes = [[DirectoryBrowserItem alloc] initWithPath:@"/Volumes/"];
		[mRootDirectories addObject:home];
		[mRootDirectories addObject:volumes];
		[home release];
		[volumes release];

		// update the view
		[mDirectoryBrowserView reloadData];
	}
	return self;
}

-(void)dealloc
{
	[mRootDirectories release];
	[super dealloc];
}

/**
	This method return the nth child of an item. If the item is nil, it's the
	nth root item.
*/
-(id)outlineView:(NSOutlineView *)outlineView child:(NSInteger)index ofItem:(id)item
{
	if (item == nil)
		return [mRootDirectories objectAtIndex:index];
	else
		return [[item children] objectAtIndex:index];
}

/**
	Returns YES if an item can be expanded (e.g. it has child(ren))
*/
-(BOOL)outlineView:(NSOutlineView *)outlineView isItemExpandable:(id)item
{
	if (item == nil || [item canExpand] != NO)
		return YES;
	return NO;
}

/**
	Returns the number of children of the given item. If item is nil, return the
	number of root items
*/
-(NSInteger)outlineView:(NSOutlineView *)outlineView numberOfChildrenOfItem:(id)item
{
	if (item == nil)
		return [mRootDirectories count];
	return [[item children] count];
}

/**
	Return the value for the given item and column. We use the column's
	identifier and use it as a key to get the corresponding value in the item.
	This way, we can add as many column as we want without touching this method.
*/
-(id)outlineView:(NSOutlineView *)outlineView objectValueForTableColumn:(NSTableColumn *)tableColumn byItem:(id)item
{
	if (item == nil)
		return @"/";
	
	return [item valueForKey:[tableColumn identifier]];
}

@end
