#import "DirectoryBrowserItem.h"
#import "../Utils/FileUtils.h"
#import "../Utils/SimpleProfiler.h"


@implementation DirectoryBrowserItem

@synthesize name		= mName;
@synthesize path		= mPath;
@synthesize numImages	= mNumImages;

-(id)initWithPath:(NSString *)path
{
	self = [super init];
	if (self != nil)
	{
		mPath		= [path copy];
		mName		= [[mPath lastPathComponent] retain];
		mChildren	= nil;
		mNumImages	= 0;
		
		PROFILING_START(@"DirectoryBrowserItem - initWithPath");
		
		// check if this item can expand here. If we do it on demand, it will
		// make scrolling lag : when an item becomes visible, the data source
		// will ask if it can expand, and reading from the disk to check if we
		// have a child directory takes some time, and scrolling becomes laggy.
		// The disadvantage here is that an item with lots of children will
		// take more time to expand, but I prefer this over a laggy scroll.
		mCanExpand	= -1;
		NSURL * url = [NSURL fileURLWithPath:mPath];
		NSFileManager * fileManager = [NSFileManager defaultManager];
		NSDirectoryEnumerator * enumerator = [fileManager enumeratorAtURL:url
														  includingPropertiesForKeys:nil
														  options:NSDirectoryEnumerationSkipsHiddenFiles
														  errorHandler:nil];
		mCanExpand = NO;
		for (NSURL * url in enumerator)
		{
			if (CFURLHasDirectoryPath((CFURLRef)url))
			{
				mCanExpand = YES;
				break;
			}
		}
		
		PROFILING_STOP();
	}
	return self;
}

-(void)dealloc
{
	[mPath release];
	[mName release];
	[mChildren release];
	[super dealloc];
}

-(BOOL)canExpand
{
	return mCanExpand;
}

-(NSMutableArray *)children
{
	// the first time children is accessed, initliaze its content
	if (mChildren == nil)
	{
		mChildren = [[NSMutableArray alloc] init];

		// get the content of the directory
		NSFileManager * fileManager = [NSFileManager defaultManager];
		NSURL * url = [NSURL fileURLWithPath:mPath];
		NSArray * content = [fileManager contentsOfDirectoryAtURL:url
										 includingPropertiesForKeys:nil
										 options:NSDirectoryEnumerationSkipsHiddenFiles
										 error:nil];

		// parse the content, count the number of images and directories.
		for (NSURL * item in content)
		{
			if (CFURLHasDirectoryPath((CFURLRef)item))
			{
				// create a child item and store it
				DirectoryBrowserItem * child = [[DirectoryBrowserItem alloc] initWithPath:[item path]];
				[mChildren addObject:child];
				[child release];
			}
			else if ([FileUtils isImage:[item path]] == YES)
			{
				// increment the number of images contained in the current directory
				++mNumImages;
			}
		}
	}
	return mChildren;
}

@end
