#import <Quartz/Quartz.h>
#import "ImageBrowserDataSource.h"
#import "ImageBrowserItem.h"
#import "../Utils/FileUtils.h"
#import "../Utils/Utils.h"


@implementation ImageBrowserDataSource

@synthesize imageBrowser = mImageBrowser;

-(id)init
{
	DEBUG_LOG(@"ImageBrowserDataSource - init");

	self = [super init];
	if (self != nil)
	{
		mCurrentDirectory = nil;
		mImages = [[NSMutableArray alloc] init];
		mImportedImages = [[NSMutableArray alloc] init];
	}
	return self;
}

-(void)dealloc
{
	[mImages release];
	[mImportedImages release];
	[super dealloc];
}

-(NSString *)currentDirectory
{
	return mCurrentDirectory;
}

-(void)setCurrentDirectory:(NSString *)path
{
	if (path != mCurrentDirectory && path != nil)
	{
		// copy the new path
		[mCurrentDirectory release];
		mCurrentDirectory = [path copy];

		// reset the selection
		[mImages removeAllObjects];

		// create a new thread to load the images and start it
		[NSThread detachNewThreadSelector:@selector(addImagesInPath:)
				  toTarget:self
				  withObject:mCurrentDirectory];
	}
}

-(void)addImagesInPath:(NSString *)path
{
	NSAutoreleasePool * pool = [[NSAutoreleasePool alloc] init];

	// get the content of the directory
	NSFileManager * fileManager = [[NSFileManager alloc] init];
	NSArray * content = [fileManager contentsOfDirectoryAtPath:path error:nil];

	// for each item, create an image object and add to the mImportedImages array
	for (NSString * p in content)
	{
		// ignore files beginning with '.'
		if ([[p lastPathComponent] characterAtIndex:0] == '.')
			continue;

		// complete file path
		NSString * file = [path stringByAppendingPathComponent:p];

		// ignore non-images files
		if ([FileUtils isImage:file] == NO)
			continue;

		ImageBrowserItem * image = [[ImageBrowserItem alloc] init];
		[image setPath:file];
		[image setTitle:[file lastPathComponent]];
		[mImportedImages addObject:image];
		[image release];
	}

	// when loading is done, update the data source in the main thread
	[self performSelectorOnMainThread:@selector(updateDatasource)
		  withObject:nil
		  waitUntilDone:NO];
	
	// cleanup
	[fileManager release];
	[pool release];
}

-(void)updateDatasource
{
	[mImages addObjectsFromArray:mImportedImages];
	[mImportedImages removeAllObjects];
	[mImageBrowser reloadData];
}

-(NSUInteger)numberOfItemsInImageBrowser:(IKImageBrowserView *)view
{
    return [mImages count];
}

-(id)imageBrowser:(IKImageBrowserView *)view itemAtIndex:(NSUInteger)index
{
    return [mImages objectAtIndex:index];
}

@end
